body {
	background-color: #252525;
	font-family: Verdana;
	border:0px;
	height: 0px;
	margin:10px;
}

p {
	white-space: pre;
	font-family: monospace;
	display: block;
}

a {
	color: blue;
	text-decoration: underline;
	cursor: pointer;
}

.header {
	width:320px;
	border: 0px;
	border-color: black;
	text-align: center;
	vertical-align: top;
}

.button {
	border: 1px solid;
	border-color: black;
	margin-right: 2px;
	width:32px;
	background-color: #a0a0a0;
	color: black;
	z-index: 10px;
}

.button:hover {
	background-color: #c0c0c0 !important;
	color: #202020 !important;
}

input {
/*	width:310px; */
	width:190px;
	border: 1px solid black;
	background-color: #707070;
	color: #202020;
}

input:focus {
	background-color:white !important;
	color:black;
}

.popup_content {
	font-size:12px;
	border: 1px solid black;
	margin-left:2px;
	padding:10px;
	width:270px;
	/* height:232px; */
	text-align:left;
	vertical-align:top;
	/* overflow:auto; */
	color: white;
	background-color:#000000;
}

.console2 {
	font-size:12px;
	width:20px;
	height:100%;
	text-align:left;
color:white;
position:fixed;
top:24;
left:0px;
background-color:black;
	/* overflow:auto; */
	/* background-color:#f0f0f0; */
	color: white;
}

.console {
	font-size:12px;
	width:310px;
	height:100%;
	text-align:left;
	left:20px;
	/* overflow:auto; */
	/* background-color:#f0f0f0; */
	color: white;
}

.popup {
	position:absolute;
	z-index: 5;
	top: 10px;
	left: 10px;
	width:300px;
	height:290px;
	border: 1px solid black;
	background-color: #505070;
}

a:link {
	color: #3096cc;
	text-decoration: none;
	background-color: transparent;
}
a:visited {
	color: #3096fc;
	text-decoration : none;
	background-color: transparent;
}
a:active {
	color: #3096fc;
	text-decoration : none;
	background-color: transparent;
}
a:hover {
	color: #3096fc;
	text-decoration : underline;
	background-color: transparent;
}

a {
	background-color: #171717 !important;
}

.popup_background {
	opacity:0.7;
	filter:alpha(opacity=70);
	position:fixed;
	top: 0px;
	left: 0px;
	width:100%;
	min-height:100%;
	border: 0px;
	background-color: #000000;
	z-index: -30;
}

.popup_title {
	margin-left: 5px;
	font-size:14px;
}

.menubar {
	position:fixed;
	top:4px;
	left:340px;
	color: white;
	font-size: 14px;
	background-color:black;
}

.editor {
	position:fixed;
	top: 22px;
	width:16px;
	height:100%;
	right:0px;
	color: white;
	font-size: 14px;
	background-color:black;
}

a.rcx {
	background-color:red !important;
}

a.rcx:hover {
	color:white !important;
	background-color:yellow ;
}

.editor_textarea {
	visibility: hidden;
	background-color:#101010;
	border-width:1px !important;
	border-color:#a0a0a0 !important;
	width:95%;
	height:90%;
	font-family: Courier;
	font-size:14px;
margin-left:10px;
	color: white;
}

.scroller {
	width:100%;
	background-color:#302050;
	color: #a0a0c0;
	cursor: pointer;
	border-color:black;
	border-width:3px !important;
}

.scroller:hover {
	width:100%;
	background-color:#404070;
	color: #808080;
	cursor: pointer;
}

.minibutton {
	background-color: #808080 !important;
	color: black !important;
	height:22px !important;
}

.minibutton:hover {
	background-color: #c0c0c0 !important;
	color: #202020 !important;
}
