.onoffswitch{position:relative;width:70px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.onoffswitch-checkbox{display:none}.onoffswitch-label{display:block;overflow:hidden;cursor:pointer;border:2px solid #999;border-radius:20px}.onoffswitch-inner{width:200%;margin-left:-100%;-webkit-transition:margin .2s ease-in;transition:margin .2s ease-in}.onoffswitch-inner:after,.onoffswitch-inner:before{float:left;padding:0;width:50%;height:26px;font:bold 14px/26px Trebuchet,Arial,sans-serif;-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box}.onoffswitch-inner:before{content:"YES";padding-left:10px;color:#fff;background-color:#2fccff}.onoffswitch-inner:after{content:"NO";padding-right:10px;color:#999;background-color:#eee;text-align:right}.onoffswitch-switch{position:absolute;top:0;bottom:0;right:40px;width:18px;margin:4px;background:#fff;border:2px solid #999;border-radius:20px;-webkit-transition:right .2s ease-in,-webkit-box-shadow .2s ease-in;transition:right .2s ease-in,box-shadow .2s ease-in}.onoffswitch-checkbox:checked+.onoffswitch-label .onoffswitch-inner{margin-left:0}.onoffswitch-checkbox:checked+.onoffswitch-label .onoffswitch-switch{right:0}