#ifndef R2_DRX_H
#define R2_DRX_H

#ifdef __cplusplus
extern "C" {
#endif

enum {
  DRX_API_LIST = 0,
  DRX_API_GET_BP = 1,
  DRX_API_SET_BP = 2,
  DRX_API_REMOVE_BP = 3,
};

#ifdef __cplusplus
}
#endif

#endif
