img{border:0}div{overflow:hidden}p{margin:0;border:0}textarea{background-color:#203040;color:#fff;border:0;padding:5px}a{color:#a06000}a:hover{color:#fff}.link{color:#fff}body,html{overflow:hidden;background-color:#000}iframe{width:100%;border:1px solid #000}@keyframes fadeInOpacity{0%{opacity:0}100%{opacity:1}}.modal{font-family:monospace;border:3px solid #000;left:10%;top:10%;right:10%;border-radius:1em;padding-top:20px;bottom:10%;background-color:#202020;color:#e0e0e0;position:absolute;z-index:10000!important}.modal_title{position:absolute;background-color:#000;color:#e0e0e0;padding:0;top:0;height:20px;font-family:monospace;width:100%;z-index:10001}.frame{font-family:monospace;border:1px solid #000;height:100%;background-color:#505050;color:#e0e0e0}.frame_title{background-color:#000;color:#fff;padding:0;top:0;height:20px;font-family:monospace;width:100%}.frame_body{background-color:#303030;overflow:scroll;height:100%}.canvas{padding-top:3px;width:100%;height:100%;background-color:#000;border-top:5px;font-size:12px;font-family:monospace}input{width:98%;border:1px solid #000;text-align:left;margin:5px;background-color:rgba(0,0,0,.3);overflow:hidden;color:#fff;font-family:monospace}h2{padding:0;color:#000;border:0;margin:10px}.minibut{text-decoration:none;font-weight:700;font-family:monospace;color:#a0a0f0}::-webkit-scrollbar{width:.5em}::-webkit-scrollbar-track{-webkit-box-shadow:inset 0 0 6px rgba(0,0,0,.3);box-shadow:inset 0 0 6px rgba(0,0,0,.3)}::-webkit-scrollbar-thumb{background-color:#a9a9a9;outline:1px solid #708090}::-webkit-scrollbar:hover{width:.7em}