let m_root=self.location.pathname.split("/").slice(0,-1).join("/");importScripts(m_root+"/r2.js");var LINES=80,MAXLINES=Math.round(1.2*LINES),TOOLONG=2*LINES*3;function appendTo(e,o){if(null!==o)for(var n=0;n<o.length;n++){var s=parseInt(o[n].offset);"0"==o[n].size||o[n].size>=TOOLONG||void 0!==e[s]&&o[n].size<=e[s]||(e[s]=parseInt(o[n].size))}}self.onmessage=function(){var o,n,e={};r2.cmdj("fj ",function(e){o=e}),r2.cmdj("aflj",function(e){n=e}),appendTo(e,o),appendTo(e,n),self.postMessage(e)};