#ifndef R_VERSION_H
#define R_VERSION_H 1
#define R_MESON_VERSION "1.12.0"
#define R2_VERSION_MAJOR 6
#define R2_VERSION_MINOR 2
#define R2_VERSION_PATCH 0
#define R2_VERSION_NUMBER 60200
#define R2_VERSION_COMMIT 37388
#define R2_VERSION "6.2.0"
#define R2_GITTAP "6.2.0"
#define R2_GITTIP "e1fc278734ad62f933fc6f91edb29e4ba732f402"
#define R2_BIRTH "2026-09-04__18:55:27"
#endif
